module ArithmeString

#Translate a string into an integer - Return nil if not a valid integer number
def ArithmeString.string_to_integer(s)
	(s && s.strip =~ /\A(\+|\-|\s*)(\s*)(\d+)\Z/) ? ($1.strip + $3).to_i : nil
end


#Translate a string into an integer value, accepting mathematical formulas applicable to integers
def ArithmeString.string_to_integer_formula(s)
	return nil if s == nil || s =~ /[^\s\+\-\*\%\/\%\(\)\d]/
	begin 
		eval "(#{s}) + 0"
	rescue Exception => detail
		nil
	end	
end

#Translate a string into a float, assumed to contain only a float - return nil if invalid
def ArithmeString.string_to_float(s)
	return nil unless s
	s = s.strip.upcase
	return nil if s == ''
	s = s.gsub(/,\d/) { |c| '.' + c[1..1] }
	return nil unless s =~ /\A(\+|\-|\s*)(\s*)(\d*)(\.?)(\d*)(e|E)?(\d*)\Z/
	r = $1.strip + $3
	[$4, $5, $6, $7].each { |p| r += p if p }
	r.to_f
end

#Translate a string to a float value, accepting mathematical forumal in the string - return nil of invalid
def ArithmeString.string_to_float_formula(s)
	return nil unless s
	s = s.strip.upcase
	lform = ["PI", "sin", "cos", "tan", "atan2", "sqrt", "log", "log10", "exp", "ceil", "floor", "round"]
	lform.each_with_index { |f, i| s = s.gsub(f.upcase, "_#{i}_") }
	s = s.gsub(/,\d/) { |c| '.' + c[1..1] }
	return nil if s =~ /[^\s\+\-\*\.\,eE_\%\/\%\(\)\d]/
	s = s.gsub(/(\A\.)\d+/) { |ss| '0' + ss }
	s = s.gsub(/(\s|\(|\+|\-)(\.)(\d+)/) { |ss| $1 + '0.' + $3 }
	s = s.gsub(',', '.').gsub(/(\d+)\.*(\d*)(e|E)?(\d*)/) { |ss| (ss =~ /\.|e|E/) ? ss : ss + '.0' }
	s = s.gsub(/_(\d+)\.\d_/) { |f| "Math::#{lform[$1.to_i]}" }
	s = s.gsub(/\s+\(/, '(')	
	begin 
		eval "0.0 + (#{s})"
	rescue Exception => detail
		nil
	end	
end

#Hold the decimal character supported by SU for length parsing
@@length_comma_or_dot = nil

#Translate a string into a Length
#The method accepts both comma and dot separator for numbers.
#return nil if invalid or the length in Inches as a Length class object)
def ArithmeString.string_to_length(s)
	return nil unless s
	s = s.strip
	return nil if s == ''
	
	#Checking whether Sketchup accepts the dot or the comma (computed only once)
	unless @@length_comma_or_dot
		begin
			"2.3".to_l
			@@length_comma_or_dot = '.'
		rescue Exception => detail
			@@length_comma_or_dot = ','
		end	
	end
	if @@length_comma_or_dot == '.'
		s = s.gsub(/\,\d/) { |c| '.' + c[1..1] }
	else	
		s = s.gsub(/\.\d/) { |c| ',' + c[1..1] }
	end
	
	#Notation with missing leading 0
	s = s.gsub(/(\A\.)\d+/) { |ss| '0' + ss }
	s = s.gsub(/(\s|\(|\+|\-)(\.)(\d+)/) { |ss| $1 + '0.' + $3 }
	
	#Simple float numbers or architectural dimensions
	unless s =~ /[^0-9.,\s'"\/]/
		begin
			return s.to_l
		rescue  Exception => detail
			return nil
		end	
	end	
	
	#Contains Units
	['cm', 'mm', 'km', 'm', 'inch', 'feet', 'mile'].each do |u|
		pat = Regexp.new '\s+' + u, Regexp::IGNORECASE
		s = s.gsub pat, '.' + u
		pat = Regexp.new '\d' + u, Regexp::IGNORECASE
		s = s.gsub(pat) { |c| c[0..0] + '.' + u }
	end
	
	#Evaluating the result
	s = s.gsub(/\,\d/) { |c| '.' + c[1..1] }
	begin
		return eval(s).inch
	rescue  Exception => detail
		return nil
	end	
end

#Translate a string into a Length, with formula
#The convention is that for * and /, factors are always in second position (i.e. 2cm * 4, NOT 4 * 2cm)
#Operators accepted are +, -, * and /
#return nil if invalid or the length in Inches as a Length class object)
def ArithmeString.string_to_length_formula(s)
	#Verifying the string and initialization
	return nil unless s
	s = s.strip
	return nil if s == ''
	s = '(' + s + ')'
	
	#Progressive parsing and transfer to string <sresult>
	plevel = []
	ffloat = false
	sresult = ""
	while s =~ /\(|\)|\*|\+|\-|\//
		sprev = $`.strip
		stoken = $&
		snext = $'
		
		#architectural units can use '/' as well
		if stoken == '/' && !ffloat && s =~ /\d+(\'+\s*|\s+)\d+\/\d+/
			stoken = nil
			sprev = $&
			snext = $'
		end
		
		#Evaluating the number, as flloat or as length
		if sprev != ''
			val = (ffloat) ? ArithmeString.string_to_float(sprev) : ArithmeString.string_to_length(sprev)
			return nil unless val
			sresult += val.to_f.to_s
		end	
		
		#Managing the parentheses and operators
		case stoken
		when '('
			plevel.push ffloat
		when ')'	
			return nil if plevel.length == 0		# No matching parenthesis
			ffloat = plevel.pop if stoken == ')'
		when '*', '/'
			ffloat = true
		when '+', '-'
			ffloat = false unless plevel.last
		end		
		sresult += stoken if stoken
		s = snext		
	end
	
	#Completing with parenthesis if missing
	sresult += ')' * plevel.length if plevel.length > 0
		
	#Evaluating the result as a length
	begin
		return eval(sresult).inch
	rescue Exception => detail
		return nil
	end	
end

end	#module ArithmeString